pub enum ShapeInferenceMode {
Off,
On,
Strict,
}Expand description
Set whether shape and type inference is run when loading a model.
Variants
Off
Do not run shape inference
On
Run shape inference in best-effort mode.
If shape inference is unsupported or fails for any operators, the model will still load but some optimizations might be missed.
Strict
Run shape inference in strict mode.
The model will fail to load if shape inference cannot infer the shapes or types of any values.